import {
  MalClientAuthentication
} from "./chunk-K2DFM6AT.js";
import {
  MalExpiredTokenError,
  MalInvalidParameterError,
  MalRateLimitError
} from "./chunk-EZCSSTDX.js";
import {
  MalApiHeader,
  MalAuthType
} from "./chunk-BCMNSRGV.js";

// lib/clients/base-mal-client.ts
import { BaseClient } from "@dvcol/base-http-client";
import { injectCorsProxyPrefix, injectUrlPrefix, parseBodyUrlEncoded, parseUrl, patchResponse } from "@dvcol/base-http-client/utils/client";
import { BaseApiHeaders, BaseHeaderContentType } from "@dvcol/base-http-client/utils/http";
import { HttpMethod } from "@dvcol/common-utils";
var parsePagingLinks = (link) => {
  const params = new URLSearchParams(link.split("?")[1] ?? link);
  return {
    offset: Number(params.get("offset")),
    limit: Number(params.get("limit"))
  };
};
var isPagedData = (data) => "paging" in data;
var parseResponse = (response) => {
  if (!isPagedData(response)) return response;
  const result = {
    data: response.data,
    pagination: {}
  };
  if (response.paging.previous) result.pagination.previous = { ...parsePagingLinks(response.paging.previous), link: response.paging.previous };
  if (response.paging.next) result.pagination.next = { ...parsePagingLinks(response.paging.next), link: response.paging.next };
  return result;
};
var patchMalResponse = (response) => patchResponse(response, parseResponse);
var BaseMalClient = class extends BaseClient {
  /**
   * Creates an instance of BaseMalClient.
   * @param options - The options for the client.
   * @param authentication - The authentication for the client.
   * @param api - The API endpoints for the client.
   */
  constructor(options, authentication = {}, api = {}) {
    const _authentication = authentication instanceof MalClientAuthentication ? authentication : new MalClientAuthentication(authentication);
    super(options, _authentication, api);
  }
  /**
   * Parses the template to construct the headers for a Mal API request.
   *
   * @protected
   *
   * @template T - The type of the parameters.
   *
   * @param {MalApiTemplate<T>} template - The template for the API endpoint.
   *
   * @returns {HeadersInit} The parsed request headers.
   *
   * @throws {Error} Throws an error if OAuth is required and the access token is missing.
   */
  _parseHeaders(template) {
    const headers = {
      [BaseApiHeaders.UserAgent]: this.settings.useragent,
      [BaseApiHeaders.ContentType]: [HttpMethod.POST, HttpMethod.PUT, HttpMethod.PATCH].map(String).includes(template.method) ? BaseHeaderContentType.FormUrlEncoded : BaseHeaderContentType.Json,
      [MalApiHeader.MalClientId]: this.settings.client_id,
      [MalApiHeader.MalApiVersion]: template.opts?.version
    };
    if (template.opts?.auth === MalAuthType.Client && !this.settings.client_id) {
      throw new MalInvalidParameterError("OAuth required: client_id is missing");
    }
    if (template.opts?.auth === MalAuthType.User) {
      if (!this.auth.access_token) throw new MalInvalidParameterError("OAuth required: access_token is missing");
      else if (this.auth.isExpired()) throw new MalExpiredTokenError("OAuth required: access_token has expired");
    }
    if (template.opts?.auth && this.auth.access_token) headers[BaseApiHeaders.Authorization] = `Bearer ${this.auth.access_token}`;
    return headers;
  }
  /**
   * Parses the parameters and constructs the URL for a Mal API request.
   *
   * @protected
   *
   * @template T - The type of the parameters.
   *
   * @param template - The template for the API endpoint.
   * @param {T} params - The parameters for the API call.
   *
   * @returns {string} The URL for the Mal API request.
   *
   * @throws {Error} Throws an error if mandatory parameters are missing or if a filter is not supported.
   */
  _parseUrl(template, params) {
    const versionedTemplate = injectUrlPrefix(`/${template.opts.version}`, template);
    if (template.opts.endpoint) return parseUrl(versionedTemplate, params, template.opts.endpoint);
    const _template = injectCorsProxyPrefix(versionedTemplate, this.settings);
    return parseUrl(_template, params, this.settings.endpoint);
  }
  /**
   * Parses body from a template and stringifies a {@link BodyInit}
   *
   * @protected
   *
   * @template T - The type of the parameters.
   *
   * @param template - The expected body structure.
   * @param {T} params - The actual parameters.
   *
   * @returns {BodyInit} The parsed request body.
   */
  // eslint-disable-next-line class-methods-use-this -- implemented from abstract class
  _parseBody(template, params) {
    return parseBodyUrlEncoded(template, params);
  }
  /**
   * Parses the response from the API before returning from the call.
   * @param response - The response from the API.
   *
   * @returns {MalApiResponse} The parsed response.
   * @protected
   */
  // eslint-disable-next-line class-methods-use-this -- implemented from abstract class
  _parseResponse(response) {
    if (response.status === 401 && response.headers.get(BaseApiHeaders.Authenticate)?.includes("token expired")) {
      throw new MalExpiredTokenError("OAuth required: access_token has expired", response);
    }
    if (response.status === 429) throw new MalRateLimitError(response.statusText, response);
    if (!response.ok || response.status >= 400) throw response;
    const parsed = patchMalResponse(response);
    const _clone = parsed.clone;
    parsed.clone = () => patchMalResponse(_clone.bind(parsed)());
    return parsed;
  }
};

export {
  parseResponse,
  BaseMalClient
};
