import { BaseClient, BaseBody } from '@dvcol/base-http-client';
import { RecursiveRecord } from '@dvcol/common-utils/common/models';
import { MalApi } from '../api/mal-api.endpoints.cjs';
import { MalApiRawPaginatedData, MalApiPaginatedData, MalApiQuery, MalApiResponse, MalClientSettings, MalClientOptions, IMalApi, MalApiParams, MalApiTemplate } from '../models/mal-client.model.cjs';
import { MalClientAuthentication, IMalClientAuthentication } from '../models/mal-authentication.model.cjs';
import '../models/mal-forum.model.cjs';
import '../models/mal-user.model.cjs';
import '../mal-manga.model-BSnTz6i5.cjs';
import '@dvcol/common-utils/http/fetch';

declare const parseResponse: <T extends RecursiveRecord = unknown>(response: T | MalApiRawPaginatedData<T>) => T | Partial<MalApiPaginatedData>;
/** Needed to type Object assignment */
interface BaseMalClient extends MalApi {
}
/**
 * Represents a Mal API client with common functionality.
 *
 * @class BaseMalClient
 */
declare class BaseMalClient extends BaseClient<MalApiQuery, MalApiResponse, MalClientSettings, MalClientAuthentication> implements MalApi {
    /**
     * Creates an instance of BaseMalClient.
     * @param options - The options for the client.
     * @param authentication - The authentication for the client.
     * @param api - The API endpoints for the client.
     */
    constructor(options: MalClientOptions, authentication?: IMalClientAuthentication, api?: IMalApi);
    /**
     * Parses the template to construct the headers for a Mal API request.
     *
     * @protected
     *
     * @template T - The type of the parameters.
     *
     * @param {MalApiTemplate<T>} template - The template for the API endpoint.
     *
     * @returns {HeadersInit} The parsed request headers.
     *
     * @throws {Error} Throws an error if OAuth is required and the access token is missing.
     */
    protected _parseHeaders<T extends MalApiParams = MalApiParams>(template: MalApiTemplate<T>): HeadersInit;
    /**
     * Parses the parameters and constructs the URL for a Mal API request.
     *
     * @protected
     *
     * @template T - The type of the parameters.
     *
     * @param template - The template for the API endpoint.
     * @param {T} params - The parameters for the API call.
     *
     * @returns {string} The URL for the Mal API request.
     *
     * @throws {Error} Throws an error if mandatory parameters are missing or if a filter is not supported.
     */
    protected _parseUrl<T extends MalApiParams = MalApiParams>(template: MalApiTemplate<T>, params: T): URL;
    /**
     * Parses body from a template and stringifies a {@link BodyInit}
     *
     * @protected
     *
     * @template T - The type of the parameters.
     *
     * @param template - The expected body structure.
     * @param {T} params - The actual parameters.
     *
     * @returns {BodyInit} The parsed request body.
     */
    protected _parseBody<T extends MalApiParams = MalApiParams>(template: BaseBody<string | keyof T>, params: T): BodyInit;
    /**
     * Parses the response from the API before returning from the call.
     * @param response - The response from the API.
     *
     * @returns {MalApiResponse} The parsed response.
     * @protected
     */
    protected _parseResponse<T extends RecursiveRecord = unknown>(response: MalApiResponse<T> | MalApiResponse<MalApiRawPaginatedData<T>>): MalApiResponse;
}

export { BaseMalClient, parseResponse };
