import { BaseMalClient } from './base-mal-client.cjs';
import * as _dvcol_base_http_client from '@dvcol/base-http-client';
import { IMalClientAuthentication, MalAuthorizeQuery, MalTokenExchangeQuery, MalClientAuthentication } from '../models/mal-authentication.model.cjs';
import { MalClientOptions, IMalApi } from '../models/mal-client.model.cjs';
import '@dvcol/common-utils/common/models';
import '../api/mal-api.endpoints.cjs';
import '../models/mal-forum.model.cjs';
import '@dvcol/common-utils/http/fetch';
import '../models/mal-user.model.cjs';
import '../mal-manga.model-BSnTz6i5.cjs';

/**
 * MalClient is a wrapper around the MalApi to provide basic authentication and state management.
 *
 * @class MalClient
 *
 * @extends {BaseMalClient}
 */
declare class MalClient extends BaseMalClient {
    /**
     * The url to redirect to after the user has authorized the application.
     */
    get redirectUri(): string;
    /**
     * Creates an instance of MalClient, with the necessary endpoints and settings.
     * @param settings - The settings for the client.
     * @param authentication - The authentication for the client.
     * @param api - The API endpoints for the client.
     */
    constructor(settings: MalClientOptions, authentication?: IMalClientAuthentication, api?: IMalApi);
    /**
     * Initiates the OAuth process by redirecting to the MyAnimeList website.
     * Users will be prompted to sign in and authorize the application.
     *
     * Once redirected back to the application, the code should be exchanged for an access token.
     *
     * @param redirect - The type of redirect to use (defaults to manual).
     * @param redirect_uri - The URL to redirect to after the user has authorized the application (defaults to client settings).
     * @param request.code_challenge - The code challenge to match against the code verifier.
     * @param request.state - The optional CSRF token to verify the state.
     * @param request - Additional parameters for the authorization request.
     * @returns A promise resolving to the response from the MyAnimeList website.
     *
     * @see [authorize]{@link https://myanimelist.net/apiconfig/references/authorization#step-2-client-requests-oauth-20-authentication}
     */
    authorize({ redirect, redirect_uri, ...request }?: MalAuthorizeQuery): Promise<_dvcol_base_http_client.TypedResponse<unknown>>;
    /**
     * Initiates the OAuth process by generating a URL to the MyAnimeList website.
     * Users will be prompted to sign in and authorize the application.
     *
     * Once redirected back to the application, the code should be exchanged for an access token with {@link exchangeCodeForToken}.
     *
     * @param redirect_uri - The URL to redirect to after the user has authorized the application (defaults to client settings).
     * @param request - Additional parameters for the authorization request.
     * @returns A promise resolving to the response from the MyAnimeList website.
     *
     * @see [authorize]{@link https://myanimelist.net/apiconfig/references/authorization#step-2-client-requests-oauth-20-authentication}
     */
    resolveAuthorizeUrl({ redirect_uri, ...request }?: Omit<MalAuthorizeQuery, 'redirect'>): Promise<string>;
    /**
     * Exchanges the authorization code obtained after the user has authorized the application with {@link authorize} or {@link resolveAuthorizeUrl}.
     *
     * @param  code - The authorization code obtained from the user.
     * @param code_verifier - The code verifier to match again the code challenge.
     * @param redirect_uri - The URL to redirect to after the user has authorized the application (defaults to client settings).
     * @param  state - The optional CSRF token to verify the state.
     *
     * @returns  A promise resolving to the Trakt authentication information.
     *
     * @throws Error Throws an error if the CSRF token is invalid.
     */
    exchangeCodeForToken({ code, code_verifier, redirect_uri }: MalTokenExchangeQuery, state?: string): Promise<MalClientAuthentication>;
    /**
     * Refreshes the access token using the refresh token.
     *
     * @returns  A promise resolving to the updated Trakt authentication information.
     * @param refresh_token - The refresh token to use (defaults to the stored refresh token).
     *
     * @throws Error Throws an error if no refresh token is found.
     */
    refreshAccessToken(refresh_token?: string): Promise<MalClientAuthentication>;
    /**
     * Imports the provided authentication information into the client.
     * If the access token is expired, it attempts to refresh it.
     *
     * @param auth - The authentication information to import.
     *
     * @returns A promise resolving to the imported authentication information.
     */
    importAuthentication(auth?: IMalClientAuthentication): Promise<MalClientAuthentication>;
}

export { MalClient };
