import { BaseSettings, BaseQuery, BaseRequest, ResponseOrTypedResponse, BaseOptions, BaseTemplateOptions, BaseTemplate, ClientEndpoint } from '@dvcol/base-http-client';
import { RecursiveRecord } from '@dvcol/common-utils/common/models';
import { CancellablePromise } from '@dvcol/common-utils/http/fetch';

declare const ApiVersion: {
    readonly v1: "v1";
    readonly v2: "v2";
};
type ApiVersions = (typeof ApiVersion)[keyof typeof ApiVersion];
/**
 * @see [documentation]{@link https://myanimelist.net/apiconfig/references/api/v2}
 */
type MalClientSettings = BaseSettings<{
    /** The client ID you received from MyAnimeList when you registered your application. */
    client_id: string;
    /** The client secret you received from MyAnimeList when you registered your application. */
    client_secret: string;
    /** URI specified in your app settings. */
    redirect_uri?: string;
    /** The consumer client identifier */
    useragent: string;
    /** The access token lifetime in seconds */
    TokenTTL: number;
    /** The refresh token lifetime in seconds */
    RefreshTokenTTL: number;
}>;
type MalApiQuery<T = unknown> = BaseQuery<BaseRequest, T>;
type MalApiResponse<T = unknown> = ResponseOrTypedResponse<T>;
type MalApiResponseErrorData = {
    message: string;
    error: string;
};
type MalApiRawPaginatedData<T = unknown> = {
    data: T;
    paging: {
        previous?: string;
        next?: string;
    };
};
type MalApiPagination = {
    /** Default: 0 */
    offset?: number;
    /** Default: 100 */
    limit?: number;
};
type MalApiPaginatedData<T = unknown> = {
    data: T;
    pagination: {
        previous?: MalApiPagination & {
            link: string;
        };
        next?: MalApiPagination & {
            link: string;
        };
    };
};
type MalApiFields<T extends RecursiveRecord> = string | (keyof T)[] | Partial<{
    [K in keyof T]: string | (keyof T[K])[];
}>;
type MalClientOptions = BaseOptions<MalClientSettings, MalApiResponse>;
type MalApiParams = RecursiveRecord;
declare const MalApiHeader: {
    readonly MalClientId: "X-MAL-CLIENT-ID";
    readonly MalApiVersion: "X-MAL-API-VERSION";
};
type MalApiHeaders = (typeof MalApiHeader)[keyof typeof MalApiHeader];
/**
 * @see [documentation]{@link https://myanimelist.net/apiconfig/references/api/v2#section/Authentication}
 */
declare const MalAuthType: {
    /**
     * Main auth (user OAuth)
     *
     * The API client can see and modify basic profile information and users' list data, post information to MyAnimelist on behalf of users.
     * @see [documentation]{@link https://myanimelist.net/apiconfig/references/api/v2#section/Authentication}
     **/
    readonly User: "main_auth";
    /**
     * When user login is not required, the X-MAL-CLIENT-ID request header can be used to authenticate the client by setting your API client ID.
     * @see [documentation]{@link https://myanimelist.net/apiconfig/references/api/v2#section/Authentication}
     */
    readonly Client: "client_auth";
    /**
     * Either {@link User} or {@link Client} auth
     * @see [documentation]{@link https://myanimelist.net/apiconfig/references/api/v2#section/Authentication}
     */
    readonly Both: "both_auth";
};
type MalAuthTypes = (typeof MalAuthType)[keyof typeof MalAuthType];
/**
 * Represents options that can be used in a Mal API template.
 */
type MalApiTemplateOptions<T extends string | number | symbol = string> = BaseTemplateOptions<T, boolean> & {
    /** The API version to use */
    version: ApiVersions;
    /** Optional endpoint override */
    endpoint?: string;
    /** If the method requires authentication */
    auth?: MalAuthTypes | false;
    /** If the method supports/requires nsfw filters */
    nsfw?: boolean;
    /** If the method supports/requires pagination */
    pagination?: {
        /** The minimum value */
        offset?: number;
        /** The maximum value **/
        limit?: number;
    } | false;
};
type MalApiTemplate<Parameter extends MalApiParams = MalApiParams> = BaseTemplate<Parameter, MalApiTemplateOptions<keyof Parameter>>;
interface MalClientEndpoint<Parameter extends MalApiParams = Record<string, never>, Response = unknown> {
    (param?: Parameter, init?: BodyInit): CancellablePromise<MalApiResponse<Response>>;
}
declare class MalClientEndpoint<Parameter extends MalApiParams = Record<string, never>, Response = unknown, Cache extends boolean = true> extends ClientEndpoint<Parameter, Response, Cache, MalApiTemplateOptions<keyof Parameter>> {
}
type IMalApi<Parameter extends MalApiParams = any, Response = unknown, Cache extends boolean = boolean> = {
    [key: string]: MalClientEndpoint<Parameter, Response, Cache> | IMalApi<Parameter>;
};

export { ApiVersion, type ApiVersions, type IMalApi, type MalApiFields, MalApiHeader, type MalApiHeaders, type MalApiPaginatedData, type MalApiPagination, type MalApiParams, type MalApiQuery, type MalApiRawPaginatedData, type MalApiResponse, type MalApiResponseErrorData, type MalApiTemplate, type MalApiTemplateOptions, MalAuthType, type MalAuthTypes, MalClientEndpoint, type MalClientOptions, type MalClientSettings };
